searchData={"items":[{"type":"module","title":"TimeQueue","doc":"Implements a timers queue based on a list of maps. The queue can be encoded\nas JSON.\n\nThe performance will be worse for large queues in regard to the previous\ngb_trees based implementation, although the difference is negligible for\nsmall queues (<= 1000 entries).\n\nAll map keys are shrinked to a single letter as this queue is intended to\nbe encoded and published to HTTP clients over the wire, mutiple times.\n\nThe queue keys are a map composed of the timestamp (`t`) of an event\nand an unique integer (`u`).\n\nNo erlang timers or processes are used, as the queue is only a\ndata structure. The advantage is that the queue can be persisted on\nstorage and keeps working after restarting the runtime. The queue\nmaintain its own list of unique integers to avoir relying on BEAM\nunique integers as they are reset on VM restarts.\n\nThe main drawback of a functional queue is that the queue entries must be\nmanually checked for expired timers.","ref":"TimeQueue.html"},{"type":"function","title":"TimeQueue.delete/2","doc":"Deletes an event from the queue and returns the new queue.\n\nIt accepts a time reference or a full event. When an event is given,\nits time reference will be used to find the event to  delete,\nmeaning the queue event will be deleted even if the value of the\npassed event was tampered.\n\nThe function does not fail if the event cannot be found and simply\nreturns the queue as-is.","ref":"TimeQueue.html#delete/2"},{"type":"function","title":"TimeQueue.delete_val/2","doc":"Deletes all entries from the queue whose values are equal to `unwanted`.","ref":"TimeQueue.html#delete_val/2"},{"type":"function","title":"TimeQueue.enqueue/3","doc":"Adds a new event to the queue with a TTL and the current system time as `now/0`.\n\nSee `enqueue/4`.","ref":"TimeQueue.html#enqueue/3"},{"type":"function","title":"TimeQueue.enqueue/4","doc":"Adds a new event to the queue with a TTL relative to the given timestamp in\nmilliseconds.\n\nReturns `{:ok, tref, new_queue}` where `tref` is a timer reference.","ref":"TimeQueue.html#enqueue/4"},{"type":"function","title":"TimeQueue.enqueue_abs/3","doc":"Adds a new event to the queue with an absolute timestamp.\n\nReturns `{:ok, tref, new_queue}` where `tref` is a timer reference.","ref":"TimeQueue.html#enqueue_abs/3"},{"type":"opaque","title":"TimeQueue.event/0","doc":"","ref":"TimeQueue.html#t:event/0"},{"type":"type","title":"TimeQueue.event_value/0","doc":"","ref":"TimeQueue.html#t:event_value/0"},{"type":"function","title":"TimeQueue.filter/2","doc":"Returns a new queue with entries for whom the given callback returned a truthy\nvalue.\n\nUse `filter_val/2` to filter only using values.","ref":"TimeQueue.html#filter/2"},{"type":"function","title":"TimeQueue.filter_val/2","doc":"Returns a new queue with entries for whom the given callback returned a truthy\nvalue.\n\nUnlinke `filter/2`, the callback is only passed the event value.","ref":"TimeQueue.html#filter_val/2"},{"type":"function","title":"TimeQueue.new/0","doc":"Creates an empty time queue.\n\n    iex> tq = TimeQueue.new()\n    iex> TimeQueue.peek_event(tq)\n    :empty","ref":"TimeQueue.html#new/0"},{"type":"function","title":"TimeQueue.now/0","doc":"This function is used internally to determine the current time when using\nfunctions `enqueue/3`, `pop/1`, `pop_event/1` and `peek_event/1`.\n\nIt is a simple alias to `:erlang.system_time(:millisecond)`. TimeQueue does\nnot use monotonic time since it already manages its own unique identifiers for\nqueue entries.","ref":"TimeQueue.html#now/0"},{"type":"function","title":"TimeQueue.peek/1","doc":"Returns the next value of the queue or a delay in milliseconds before the next\nvalue.\n\nSee `peek/2`.","ref":"TimeQueue.html#peek/1"},{"type":"function","title":"TimeQueue.peek/2","doc":"Returns the next value of the queue, or a delay, according to the given\ncurrent time in milliseconds.\n\nJust like `pop/2` _vs._ `pop_event/2`, `peek` wil only return `{:ok, value}`\nwhen a timeout is reached whereas `peek_event` will return `{:ok, event}`.","ref":"TimeQueue.html#peek/2"},{"type":"function","title":"TimeQueue.peek_event/1","doc":"Returns the next event of the queue or a delay in milliseconds before the next\nvalue.\n\nevent values can be retrieved with `TimeQueue.value/1`.\n\nSee `peek_event/2`.","ref":"TimeQueue.html#peek_event/1"},{"type":"function","title":"TimeQueue.peek_event/2","doc":"Returns the next event of the queue according to the given current time in\nmilliseconds.\n\nPossible return values are:\n\n- `:empty`\n- `{:ok, event}` if the timestamp of the first event is `<=` to the given\n  current time.\n- `{:delay, tref, ms}` if the timestamp of the first event is `>` to the given\n  current time. The remaining amount of milliseconds is returned.","ref":"TimeQueue.html#peek_event/2"},{"type":"function","title":"Example - TimeQueue.peek_event/2","doc":"iex> {:ok, tref, tq} = TimeQueue.new() |> TimeQueue.enqueue(100, :hello, _now = 0)\n    iex> {:delay, ^tref, 80} = TimeQueue.peek_event(tq, _now = 20)\n    iex> {:ok, _} = TimeQueue.peek_event(tq, _now = 100)","ref":"TimeQueue.html#peek_event/2-example"},{"type":"function","title":"TimeQueue.pop/1","doc":"Extracts the next event in the queue or returns a delay.\n\nSee `pop/2`.","ref":"TimeQueue.html#pop/1"},{"type":"function","title":"TimeQueue.pop/2","doc":"Extracts the next event in the queue according to the given current time in\nmilliseconds.\n\nMuch like `pop_event/2` but the tuple returned when an event time is reached\n(returns with `:ok`) success will only contain the value inserted in the\nqueue.\n\nPossible return values are:\n\n- `:empty`\n- `{:ok, value, new_queue}` if the timestamp of the first event is `<=` to the\n  given current time. The event is deleted from `new_queue`.\n- `{:delay, tref, ms}` if the timestamp of the first event is `>` to the given\n  current time. The remaining amount of milliseconds is returned.","ref":"TimeQueue.html#pop/2"},{"type":"function","title":"Example - TimeQueue.pop/2","doc":"iex> {:ok, tref, tq} = TimeQueue.new() |> TimeQueue.enqueue(100, :hello, _now = 0)\n    iex> {:delay, ^tref, 80} = TimeQueue.pop(tq, _now = 20)\n    iex> {:ok, value, _} = TimeQueue.pop(tq, _now = 100)\n    iex> value\n    :hello","ref":"TimeQueue.html#pop/2-example"},{"type":"function","title":"TimeQueue.pop_event/1","doc":"Extracts the next event in the queue with the current system time as `now/0`.\n\nSee `pop_event/2`.","ref":"TimeQueue.html#pop_event/1"},{"type":"function","title":"TimeQueue.pop_event/2","doc":"Extracts the next event in the queue according to the given current time in\nmilliseconds.\n\nPossible return values are:\n\n- `:empty`\n- `{:ok, event, new_queue}` if the timestamp of the first event is `<=` to the\n  given current time. The event is deleted from `new_queue`.\n- `{:delay, tref, ms}` if the timestamp of the first event is `>` to the given\n  current time. The remaining amount of milliseconds is returned.","ref":"TimeQueue.html#pop_event/2"},{"type":"function","title":"Example - TimeQueue.pop_event/2","doc":"iex> {:ok, tref, tq} = TimeQueue.new() |> TimeQueue.enqueue(100, :hello, _now = 0)\n    iex> {:delay, ^tref, 80} = TimeQueue.pop_event(tq, _now = 20)\n    iex> {:ok, event, _} = TimeQueue.pop_event(tq, _now = 100)\n    iex> TimeQueue.value(event)\n    :hello","ref":"TimeQueue.html#pop_event/2-example"},{"type":"function","title":"TimeQueue.size/1","doc":"Returns the number of entries in the queue.","ref":"TimeQueue.html#size/1"},{"type":"function","title":"TimeQueue.stream/1","doc":"Returns a stream of all the events in the queue.\n\nNote that this stream is immediate and does not wait for events.","ref":"TimeQueue.html#stream/1"},{"type":"opaque","title":"TimeQueue.t/0","doc":"","ref":"TimeQueue.html#t:t/0"},{"type":"function","title":"TimeQueue.timeout/1","doc":"Returns the time to next event.\n\nSame as `timeout(tq, :infinity, now())`. See `timeout/3`.","ref":"TimeQueue.html#timeout/1"},{"type":"function","title":"TimeQueue.timeout/2","doc":"Returns the time to next event.\n\nAccepts either the current time or an atom to return when the queue is empty.\n\n* If an integer is given, it will has the same result as `timeout(tq, :infinity, integer)`.\n* If an atom is given, it will has the same result as `timeout(tq, atom, now())`.\n\nSee `timeout/3`.","ref":"TimeQueue.html#timeout/2"},{"type":"function","title":"TimeQueue.timeout/3","doc":"Provides a `GenServer` compatible timeout from the queue, giving the time to\nthe next event.\n\nAccepts the current time as a third argument or will default to the current\nsystem time.\n\nReturns:\n* `if_empty` when the queue is empty. A common value in that case is either\n  `:infinity` or `:hibernate`.\n* `0` when there the next event time has been reached.\n* The delay to the next event otherwise.\n\nFor instance, when used in a `GenServer`, this function can tell when to\nautomatically wakeup the process.\n\n    def handle_call(request, _from, state) do\n      state = do_stuff(state, request)\n      timeout = TimeQueue.timeout(state.time_queue)\n      {:reply, :ok, state, timeout}\n    end\n\n  By default, `timeout/1` and `timeout/2` (if called with an integer for the\n  current time) will return `:infinity`  if the queue if empty. But you may\n  pass `:hibernate` as the second argument for `timeout/2` or `timeout/3` to\n  return `:hibernate` instead.\n\n  Note that any other atom than `:infinity` or `:hibernate` will be returned\n  as-is if as well if the queue is empty.","ref":"TimeQueue.html#timeout/3"},{"type":"type","title":"TimeQueue.timespec/0","doc":"","ref":"TimeQueue.html#t:timespec/0"},{"type":"type","title":"TimeQueue.timespec_unit/0","doc":"","ref":"TimeQueue.html#t:timespec_unit/0"},{"type":"function","title":"TimeQueue.timestamp/1","doc":"Returns the absolute timestamp a queue event is scheduled for.\n    iex> tq = TimeQueue.new()\n    iex> {:ok, _, tq} = TimeQueue.enqueue_abs(tq, 1234, :my_value)\n    iex> {:ok, event} = TimeQueue.peek_event(tq)\n    iex> TimeQueue.timestamp(event)\n    1234","ref":"TimeQueue.html#timestamp/1"},{"type":"type","title":"TimeQueue.timestamp_ms/0","doc":"","ref":"TimeQueue.html#t:timestamp_ms/0"},{"type":"opaque","title":"TimeQueue.tref/0","doc":"","ref":"TimeQueue.html#t:tref/0"},{"type":"function","title":"TimeQueue.tref/1","doc":"Returns the time reference of a queue event. This reference is\nused as a key to identify a unique event.\n    iex> tq = TimeQueue.new()\n    iex> {:ok, tref, tq} = TimeQueue.enqueue(tq, 10, :my_value)\n    iex> Process.sleep(10)\n    iex> {:ok, event} = TimeQueue.peek_event(tq)\n    iex> tref == TimeQueue.tref(event)\n    true","ref":"TimeQueue.html#tref/1"},{"type":"type","title":"TimeQueue.ttl/0","doc":"","ref":"TimeQueue.html#t:ttl/0"},{"type":"function","title":"TimeQueue.value/1","doc":"Returns the value of a queue event.\n    iex> tq = TimeQueue.new()\n    iex> {:ok, _, tq} = TimeQueue.enqueue(tq, 10, :my_value)\n    iex> Process.sleep(10)\n    iex> {:ok, event} = TimeQueue.peek_event(tq)\n    iex> TimeQueue.value(event)\n    :my_value","ref":"TimeQueue.html#value/1"},{"type":"module","title":"TimeQueue.GbTrees","doc":"Implements a timers queue based on [gb_trees](http://erlang.org/doc/man/gb_trees.html).\n\nThe queue keys are a two-tuple composed of the timestamp of an event\nand an unique integer.\n\nNo erlang timers or processes are used, as the queue is only a\ndata structure. The advantage is that the queue can be persisted on\nstorage and keep working after restarting the runtime. The queue\nmaintain its own list of unique integers to avoir relying on BEAM\nunique integers as they are reset on restart.\n\nThe main drawback is that the queue entries must be manually checked\nfor expired timers.","ref":"TimeQueue.GbTrees.html"},{"type":"function","title":"TimeQueue.GbTrees.delete/2","doc":"Deletes an event from the queue and returns the new queue.\n\nIt accepts a time reference or a full event. When an event is given,\nits time reference will be used to find the event to  delete,\nmeaning the queue event will be deleted even if the value of the\npassed event was tampered.\n\nThe function does not fail if the event cannot be found and simply\nreturns the queue as-is.","ref":"TimeQueue.GbTrees.html#delete/2"},{"type":"function","title":"TimeQueue.GbTrees.delete_val/2","doc":"Deletes all entries from the queue whose values are equal to `unwanted`.\n\nThis function is slow with `gb_trees`, see `filter/2`.","ref":"TimeQueue.GbTrees.html#delete_val/2"},{"type":"function","title":"TimeQueue.GbTrees.enqueue/3","doc":"Adds a new event to the queue with a TTL and the current system time as `now/0`.\n\nSee `enqueue/4`.","ref":"TimeQueue.GbTrees.html#enqueue/3"},{"type":"function","title":"TimeQueue.GbTrees.enqueue/4","doc":"Adds a new event to the queue with a TTL relative to the given timestamp in\nmilliseconds.\n\nReturns `{:ok, tref, new_queue}` where `tref` is a timer reference.","ref":"TimeQueue.GbTrees.html#enqueue/4"},{"type":"function","title":"TimeQueue.GbTrees.enqueue_abs/3","doc":"Adds a new event to the queue with an absolute timestamp.\n\nReturns `{:ok, tref, new_queue}` where `tref` is a timer reference.","ref":"TimeQueue.GbTrees.html#enqueue_abs/3"},{"type":"opaque","title":"TimeQueue.GbTrees.event/0","doc":"","ref":"TimeQueue.GbTrees.html#t:event/0"},{"type":"type","title":"TimeQueue.GbTrees.event_value/0","doc":"","ref":"TimeQueue.GbTrees.html#t:event_value/0"},{"type":"function","title":"TimeQueue.GbTrees.filter/2","doc":"Returns a new queue with entries for whom the given callback returned a truthy\nvalue.\n\nWith the gb_trees implementation, this operation is _very_ expensive as we\nconvert the tree to and ordered list, filter the list, and convert back to a\ntree.","ref":"TimeQueue.GbTrees.html#filter/2"},{"type":"function","title":"TimeQueue.GbTrees.filter_val/2","doc":"Returns a new queue with entries for whom the given callback returned a truthy\nvalue.\n\nUnlinke `filter/2`, the callback is only passed the event value.\n\nThis function is slow with `gb_trees`, see `filter/2`.","ref":"TimeQueue.GbTrees.html#filter_val/2"},{"type":"opaque","title":"TimeQueue.GbTrees.id/0","doc":"","ref":"TimeQueue.GbTrees.html#t:id/0"},{"type":"function","title":"TimeQueue.GbTrees.new/0","doc":"Creates an empty time queue.\n\n    iex> tq = TimeQueue.GbTrees.new()\n    iex> TimeQueue.GbTrees.peek_event(tq)\n    :empty","ref":"TimeQueue.GbTrees.html#new/0"},{"type":"function","title":"TimeQueue.GbTrees.peek/1","doc":"Returns the next value of the queue or a delay in milliseconds before the next\nvalue.\n\nSee `peek/2`.","ref":"TimeQueue.GbTrees.html#peek/1"},{"type":"function","title":"TimeQueue.GbTrees.peek/2","doc":"Returns the next value of the queue, or a delay, according to the given\ncurrent time in milliseconds.\n\nJust like `pop/2` _vs._ `pop_event/2`, `peek` wil only return `{:ok, value}`\nwhen a timeout is reached whereas `peek_event` will return `{:ok, event}`.","ref":"TimeQueue.GbTrees.html#peek/2"},{"type":"function","title":"TimeQueue.GbTrees.peek_event/1","doc":"Returns the next event of the queue or a delay in milliseconds before the next\nvalue.\n\nevent values can be retrieved with `TimeQueue.GbTrees.value/1`.\n\nSee `peek_event/2`.","ref":"TimeQueue.GbTrees.html#peek_event/1"},{"type":"function","title":"TimeQueue.GbTrees.peek_event/2","doc":"Returns the next event of the queue according to the given current time in\nmilliseconds.\n\nPossible return values are:\n\n- `:empty`\n- `{:ok, event}` if the timestamp of the first event is `<=` to the given\n  current time.\n- `{:delay, tref, ms}` if the timestamp of the first event is `>` to the given\n  current time. The remaining amount of milliseconds is returned.","ref":"TimeQueue.GbTrees.html#peek_event/2"},{"type":"function","title":"Example - TimeQueue.GbTrees.peek_event/2","doc":"iex> {:ok, tref, tq} = TimeQueue.GbTrees.new() |> TimeQueue.GbTrees.enqueue(100, :hello, _now = 0)\n    iex> {:delay, ^tref, 80} = TimeQueue.GbTrees.peek_event(tq, _now = 20)\n    iex> {:ok, _} = TimeQueue.GbTrees.peek_event(tq, _now = 100)","ref":"TimeQueue.GbTrees.html#peek_event/2-example"},{"type":"function","title":"TimeQueue.GbTrees.pop/1","doc":"Extracts the next event in the queue or returns a delay.\n\nSee `pop/2`.","ref":"TimeQueue.GbTrees.html#pop/1"},{"type":"function","title":"TimeQueue.GbTrees.pop/2","doc":"Extracts the next event in the queue according to the given current time in\nmilliseconds.\n\nMuch like `pop_event/2` but the tuple returned when an event time is reached\n(returns with `:ok`) success will only contain the value inserted in the\nqueue.\n\nPossible return values are:\n\n- `:empty`\n- `{:ok, value, new_queue}` if the timestamp of the first event is `<=` to the\n  given current time. The event is deleted from `new_queue`.\n- `{:delay, tref, ms}` if the timestamp of the first event is `>` to the given\n  current time. The remaining amount of milliseconds is returned.","ref":"TimeQueue.GbTrees.html#pop/2"},{"type":"function","title":"Example - TimeQueue.GbTrees.pop/2","doc":"iex> {:ok, tref, tq} = TimeQueue.GbTrees.new() |> TimeQueue.GbTrees.enqueue(100, :hello, _now = 0)\n    iex> {:delay, ^tref, 80} = TimeQueue.GbTrees.pop(tq, _now = 20)\n    iex> {:ok, value, _} = TimeQueue.GbTrees.pop(tq, _now = 100)\n    iex> value\n    :hello","ref":"TimeQueue.GbTrees.html#pop/2-example"},{"type":"function","title":"TimeQueue.GbTrees.pop_event/1","doc":"Extracts the next event of the queue with the current system time as `now/0`.\n\nSee `pop_event/2`.","ref":"TimeQueue.GbTrees.html#pop_event/1"},{"type":"function","title":"TimeQueue.GbTrees.pop_event/2","doc":"Extracts the next event of the queue according to the given current time in\nmilliseconds.\n\nPossible return values are:\n\n- `:empty`\n- `{:ok, event, new_queue}` if the timestamp of the first event is `<=` to the\n  given current time. The event is deleted from `new_queue`.\n- `{:delay, tref, ms}` if the timestamp of the first event is `>` to the given\n  current time. The remaining amount of milliseconds is returned.","ref":"TimeQueue.GbTrees.html#pop_event/2"},{"type":"function","title":"Example - TimeQueue.GbTrees.pop_event/2","doc":"iex> {:ok, tref, tq} = TimeQueue.GbTrees.new() |> TimeQueue.GbTrees.enqueue(100, :hello, _now = 0)\n    iex> {:delay, ^tref, 80} = TimeQueue.GbTrees.pop_event(tq, _now = 20)\n    iex> {:ok, _, _} = TimeQueue.GbTrees.pop_event(tq, _now = 100)","ref":"TimeQueue.GbTrees.html#pop_event/2-example"},{"type":"function","title":"TimeQueue.GbTrees.size/1","doc":"Returns the number of entries in the queue.","ref":"TimeQueue.GbTrees.html#size/1"},{"type":"function","title":"TimeQueue.GbTrees.stream/1","doc":"Returns a stream of all the events in the queue.\n\nNote that this stream is immediate and does not wait for events.","ref":"TimeQueue.GbTrees.html#stream/1"},{"type":"opaque","title":"TimeQueue.GbTrees.t/0","doc":"","ref":"TimeQueue.GbTrees.html#t:t/0"},{"type":"function","title":"TimeQueue.GbTrees.timeout/1","doc":"Returns the time to next event.\n\nSame as `timeout(tq, :infinity, now())`. See `timeout/3`.","ref":"TimeQueue.GbTrees.html#timeout/1"},{"type":"function","title":"TimeQueue.GbTrees.timeout/2","doc":"Returns the time to next event.\n\nAccepts either the current time or an atom to return when the queue is empty.\n\n* If an integer is given, it will has the same result as `timeout(tq, :infinity, integer)`.\n* If an atom is given, it will has the same result as `timeout(tq, atom, now())`.\n\nSee `timeout/3`.","ref":"TimeQueue.GbTrees.html#timeout/2"},{"type":"function","title":"TimeQueue.GbTrees.timeout/3","doc":"Provides a `GenServer` compatible timeout from the queue, giving the time to\nthe next event.\n\nAccepts the current time as a third argument or will default to the current\nsystem time.\n\nReturns:\n* `if_empty` when the queue is empty. A common value in that case is either\n  `:infinity` or `:hibernate`.\n* `0` when there the next event time has been reached.\n* The delay to the next event otherwise.\n\nFor instance, when used in a `GenServer`, this function can tell when to\nautomatically wakeup the process.\n\n    def handle_call(request, _from, state) do\n      state = do_stuff(state, request)\n      timeout = TimeQueue.GbTrees.timeout(state.time_queue)\n      {:reply, :ok, state, timeout}\n    end\n\n  By default, `timeout/1` and `timeout/2` (if called with an integer for the\n  current time) will return `:infinity`  if the queue if empty. But you may\n  pass `:hibernate` as the second argument for `timeout/2` or `timeout/3` to\n  return `:hibernate` instead.\n\n  Note that any other atom than `:infinity` or `:hibernate` will be returned\n  as-is if as well if the queue is empty.","ref":"TimeQueue.GbTrees.html#timeout/3"},{"type":"type","title":"TimeQueue.GbTrees.timespec/0","doc":"","ref":"TimeQueue.GbTrees.html#t:timespec/0"},{"type":"type","title":"TimeQueue.GbTrees.timespec_unit/0","doc":"","ref":"TimeQueue.GbTrees.html#t:timespec_unit/0"},{"type":"function","title":"TimeQueue.GbTrees.timestamp/1","doc":"Returns the absolute timestamp a queue event is scheduled for.\n    iex> tq = TimeQueue.GbTrees.new()\n    iex> {:ok, _, tq} = TimeQueue.GbTrees.enqueue_abs(tq, 1234, :my_value)\n    iex> {:ok, event} = TimeQueue.GbTrees.peek_event(tq)\n    iex> TimeQueue.GbTrees.timestamp(event)\n    1234","ref":"TimeQueue.GbTrees.html#timestamp/1"},{"type":"type","title":"TimeQueue.GbTrees.timestamp_ms/0","doc":"","ref":"TimeQueue.GbTrees.html#t:timestamp_ms/0"},{"type":"opaque","title":"TimeQueue.GbTrees.tref/0","doc":"","ref":"TimeQueue.GbTrees.html#t:tref/0"},{"type":"function","title":"TimeQueue.GbTrees.tref/1","doc":"Returns the time reference of a queue event. This reference is\nused as a key to identify a unique event.\n    iex> tq = TimeQueue.GbTrees.new()\n    iex> {:ok, tref, tq} = TimeQueue.GbTrees.enqueue(tq, 10, :my_value)\n    iex> Process.sleep(10)\n    iex> {:ok, event} = TimeQueue.GbTrees.peek_event(tq)\n    iex> tref == TimeQueue.GbTrees.tref(event)\n    true","ref":"TimeQueue.GbTrees.html#tref/1"},{"type":"type","title":"TimeQueue.GbTrees.ttl/0","doc":"","ref":"TimeQueue.GbTrees.html#t:ttl/0"},{"type":"function","title":"TimeQueue.GbTrees.value/1","doc":"Returns the value of a queue event.\n    iex> tq = TimeQueue.GbTrees.new()\n    iex> {:ok, _, tq} = TimeQueue.GbTrees.enqueue(tq, 10, :my_value)\n    iex> Process.sleep(10)\n    iex> {:ok, event} = TimeQueue.GbTrees.peek_event(tq)\n    iex> TimeQueue.GbTrees.value(event)\n    :my_value","ref":"TimeQueue.GbTrees.html#value/1"},{"type":"module","title":"TimeQueue.TimeInterval","doc":"Utility to parse and format time intervals represented as strings.\n\nA time interval is formatted as pairs of numbers and units. For instance\n`1d2h` is two pairs and means 1 day and 2 hours.\n\nThe unit is always lowercase and one of:\n\n* `d` for day\n* `h` for hour\n* `m` for minute\n* `s` for second\n\nNote that the `ms` unit is not supported on parsing, but will be returned when\nformatting an interval (integer or struct) that is smaller than 1 second.\n\nThere are a couple of rules regarding the format:\n\n* The order of the pairs is not significant. `1d2h` is the same as `2h1d`.\n* The addition is supported, _i.e._ the units are cumulative. `1d1d` is the\n  same as `2d`.\n* The subtraction is not supported, each pair is absolute and just represent a\n  quantity of milliseconds. But it is possible to prefix the expression with a\n  `-`, which will make the parser return a negative value.  So `-1d1h` will\n  return a negative number, but `1d-1h` or `1d-1d` are invalid.","ref":"TimeQueue.TimeInterval.html"},{"type":"function","title":"TimeQueue.TimeInterval.days/1","doc":"Returns the number of milliseconds in `n` days.","ref":"TimeQueue.TimeInterval.html#days/1"},{"type":"function","title":"TimeQueue.TimeInterval.hours/1","doc":"Returns the number of milliseconds in `n` hours. Alias for `:timer.hours/1`.","ref":"TimeQueue.TimeInterval.html#hours/1"},{"type":"function","title":"TimeQueue.TimeInterval.minutes/1","doc":"Returns the number of milliseconds in `n` minutes. Alias for `:timer.minutes/1`.","ref":"TimeQueue.TimeInterval.html#minutes/1"},{"type":"function","title":"TimeQueue.TimeInterval.parse/1","doc":"Returns a `%TimeQueue.TimeInterval{}` struct from a time interval string.","ref":"TimeQueue.TimeInterval.html#parse/1"},{"type":"function","title":"Examples - TimeQueue.TimeInterval.parse/1","doc":"iex> TimeInterval.parse(\"1d1h\")\n    {:ok, %TimeQueue.TimeInterval{ms: 90000000}}\n\n    iex> TimeInterval.parse(\"some gibberish\")\n    {:error, {:cannot_parse_interval, \"some gibberish\"}}","ref":"TimeQueue.TimeInterval.html#parse/1-examples"},{"type":"function","title":"TimeQueue.TimeInterval.parse!/1","doc":"Same as `parse/1` but returns the struct directly or raises an\n`ArgumentError`.","ref":"TimeQueue.TimeInterval.html#parse!/1"},{"type":"function","title":"Example - TimeQueue.TimeInterval.parse!/1","doc":"iex> TimeInterval.parse!(\"1d1h\")\n    %TimeQueue.TimeInterval{ms: 90000000}","ref":"TimeQueue.TimeInterval.html#parse!/1-example"},{"type":"function","title":"TimeQueue.TimeInterval.seconds/1","doc":"Returns the number of milliseconds in `n` seconds. Alias for `:timer.seconds/1`.","ref":"TimeQueue.TimeInterval.html#seconds/1"},{"type":"type","title":"TimeQueue.TimeInterval.t/0","doc":"","ref":"TimeQueue.TimeInterval.html#t:t/0"},{"type":"function","title":"TimeQueue.TimeInterval.to_ms/1","doc":"Returns the number of milliseconds in the given interval, wrapped in an\n`:ok` tuple, or an `:error` tuple. The interval can be a string or a\n`%TimeQueue.TimeInterval{}` struct.\n\nFor convenience reasons, this function also accepts integers, which are\nreturned as is.","ref":"TimeQueue.TimeInterval.html#to_ms/1"},{"type":"function","title":"Examples - TimeQueue.TimeInterval.to_ms/1","doc":"iex> TimeInterval.to_ms(\"1d1h\")\n    {:ok, 90000000}\n\n    iex> \"1d1h\" |> TimeInterval.parse!() |> TimeInterval.to_ms()\n    {:ok, 90000000}\n\n    iex> TimeInterval.to_ms(\"some gibberish\")\n    {:error, {:cannot_parse_interval, \"some gibberish\"}}\n\n    iex> TimeInterval.to_ms(1234)\n    {:ok, 1234}","ref":"TimeQueue.TimeInterval.html#to_ms/1-examples"},{"type":"function","title":"TimeQueue.TimeInterval.to_ms!/1","doc":"Same as `to_ms/1` but returns the number of milliseconds directly or raises an\n`ArgumentError`.","ref":"TimeQueue.TimeInterval.html#to_ms!/1"},{"type":"function","title":"TimeQueue.TimeInterval.to_string/1","doc":"Returns the string representation of the given time interval. Note that as\nthis module is intended to work with long durations, the number of remaining\nmilliseconds after having computed the days, hours, minutes and seconds, is\ndiscarded.","ref":"TimeQueue.TimeInterval.html#to_string/1"},{"type":"function","title":"Examples - TimeQueue.TimeInterval.to_string/1","doc":"iex> \"1d1h\" |> TimeInterval.parse!() |> TimeInterval.to_string()\n    \"1d1h\"\n\n    iex> \"1d1d1d\" |> TimeInterval.parse!() |> TimeInterval.to_string()\n    \"3d\"\n\n    iex> TimeInterval.to_string(90000000)\n    \"1d1h\"\n\n    iex> TimeInterval.to_string(90000123) # Discared remaining milliseconds\n    \"1d1h\"\n\n    iex> TimeInterval.to_string(123) # Smaller than 1 second\n    \"123ms\"","ref":"TimeQueue.TimeInterval.html#to_string/1-examples"},{"type":"function","title":"TimeQueue.TimeInterval.to_string/2","doc":"Returns the string representation with the same rules as in `to_string/1` but\nin long form.","ref":"TimeQueue.TimeInterval.html#to_string/2"},{"type":"function","title":"Example - TimeQueue.TimeInterval.to_string/2","doc":"iex> \"1d1h\" |> TimeInterval.parse!() |> TimeInterval.to_string(:verbose)\n    \"1 day 1 hour\"","ref":"TimeQueue.TimeInterval.html#to_string/2-example"},{"type":"extras","title":"TimeQueue","doc":"# TimeQueue\n\nThis library implements a pure functional queue of timers that is persistable as\na simple erlang term. No processes or Erlang timers are used.","ref":"readme.html"},{"type":"extras","title":"Use case - TimeQueue","doc":"- When you need to attach timers to a persistent data structure (ecto schemas,\npersistent GenServers, …), for example in a board game.\n- When you need to publish timers over an API (JSON, XML, …).","ref":"readme.html#use-case"},{"type":"extras","title":"Changelog - TimeQueue","doc":"Please see the changelog on [Github (master branch)](https://github.com/lud/time_queue/blob/master/CHANGELOG.md).","ref":"readme.html#changelog"},{"type":"extras","title":"Installation - TimeQueue","doc":"```elixir\ndef deps do\n  [\n    {:time_queue, \"~> 1.1\"},\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Basic Usage - TimeQueue","doc":"```elixir\ntq = TimeQueue.new()\n{:ok, tref, tq} = TimeQueue.enqueue(tq, {500, :ms}, :myval)\n{:delay, ^tref, _delay} = TimeQueue.peek(tq)\n{:delay, ^tref, _delay} = TimeQueue.pop(tq)\n\nProcess.sleep(delay)\n\n# PEEK\n{:ok, entry} = TimeQueue.peek(tq)\n:myval = TimeQueue.value(entry)\n\n# POP\n{:ok, entry, tq} = TimeQueue.pop(tq)\n:myval = TimeQueue.value(entry)\n\n:empty = TimeQueue.pop(tq)\n```","ref":"readme.html#basic-usage"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}